// Header:
// File Name:					mypotmet.h
// File Description:
// Author:						Stanislav Subrt
// Date:							2013
// Todo:
// Notes:

#ifndef	mypotmet_h
#define mypotmet_h



//////////////////////////////////////////////////////////////////////
////////// CONFIG PART  //////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////


#define VALBUFFERSIZE	2000			//short AdcBuffer[VALBUFFERSIZE];     adc value storage during one measurement






//define number of phases in each single measurement - see phaselist below
#define PHASES_IN_MEASUREMENT	4

//define number of measurements in one cycle
#define MAXIMUM_MEASUREMENTS_IN_CYCLE	20


////////////////////////////////////////////////////////////////////
////////// TYPE DEFINITIONS //////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

typedef struct{
	float Value;
	char IsValid;
} pm_FloatValueTd;



typedef struct{
	pm_FloatValueTd PolP;
	pm_FloatValueTd PolM;
	pm_FloatValueTd Sub;
} pm_ChannelDataTd;



typedef struct{
	pm_ChannelDataTd Ch1;
	pm_ChannelDataTd Ch2;
} pm_SingleMeasDataTd;



typedef struct {
	int Polarity;							//syndet polarity +-
	int Channel;							//1,2,3,4
	int Delay;								//wait for time constants of device after change of parameters at the beggining of each phase in ms

	short *pAdcBuffer;						//pointer to buffer with adc values
	int	Samples;							//how many values to take
	int DelayBetweenSamples;				//delay between samples
	void(*PhaseCallback)(void);					//function called from main after measurement is done
} pm_PhaseInfoTd;



typedef struct {
	pm_PhaseInfoTd* PhaseList;
	int Period;								//period of syn detector
	int ActivePhaseIndex;					//for internal purposes - points to active phase
	int WaitingForCallbackHandling;			//indicates wheather phase is waiting for callback invoke
	int IsStopRequested;
} pm_MeasInfoTd;



typedef struct{
	//double MeasResultsArray[ sizeof(Method1PhaseList) / sizeof(Method1PhaseList[0]) ];
	int ReadyIndex;
	float MeasResultsArray[ PHASES_IN_MEASUREMENT ];
} pm_MeasResultsTd;


typedef struct{
	int Index;
	pm_MeasResultsTd ResultsList[MAXIMUM_MEASUREMENTS_IN_CYCLE];
} pm_CycleResultTd;


typedef struct{
	int MeasurementsInCycle;
	pm_MeasInfoTd* InfoStruct;
	pm_CycleResultTd* ResultsStruct;
	void(*CycleReadyCallback)(void);					//function called from main after whole measurement cycle is ready
	int WaitingForCallbackHandling;
	int IsReady;

} pm_MeasCycleTd;



//////////////////////////////////////////////////////////////////////
////////// EXTERNAL VARIABLES AND DEFINES/////////////////////////////
//////////////////////////////////////////////////////////////////////

#define POTMET_CYCLE_NOT_RUNNING		-2
#define POTMET_CYCLE_ALREADY_RUNNING	-1
#define POTMET_CYCLE_OK	0



extern pm_MeasCycleTd  pm_DCTestMeasCycle;
extern pm_MeasCycleTd  pm_DCRefMeasCycle;
extern pm_MeasCycleTd  pm_DCCrackDetectMeasCycle;
extern pm_MeasCycleTd  pm_DCCrackLengthMeasCycle;

extern pm_SingleMeasDataTd pm_CrackReferenceMeasurementData;
extern pm_SingleMeasDataTd pm_CrackDetectionData;
extern pm_SingleMeasDataTd pm_CrackMeasData;

//////////////////////////////////////////////////////////////////////
////////// EXTERNAL FUNCTION DECLARATIONS ////////////////////////////
//////////////////////////////////////////////////////////////////////

extern void pm_Init(void);
extern void pm_InvokeCallbacks(void);


extern int pm_StartMeasCycle(pm_MeasCycleTd* cycleInfoStruct);
extern int pm_StopMeasCycle(void);

extern char* pm_GetApertureStatus(void);
extern int pm_GetRunningCycleDuration(void);
extern int pm_GetRunningCycleProgress(void);


extern void pm_SendLatestCycleResults(pm_MeasCycleTd* cycleInfoStruct);
extern void pm_ClearLatestCycleData(void);


extern pm_MeasCycleTd* pm_GetRunningCycle(void);
extern pm_MeasCycleTd* pm_GetLatestCycle(void);


extern pm_MeasResultsTd* pm_GetSelectedMeasurementResults(pm_MeasCycleTd* cycleInfoStruct, int measurementIndex);
extern pm_MeasResultsTd* pm_GetCurrentMeasurementResults(pm_MeasCycleTd* cycleInfoStruct);

extern void pm_ClearSingleMeasurementStruct(pm_SingleMeasDataTd *dest);

extern void pm_SelectChannel(int ch);
extern void pm_SendLatestResultsTest(void);

#endif	//mypotmet_h
