// Header:
// File Name:					myadc.c
// File Description:
// Author:						Stanislav Subrt
// Date:							2013
// Todo:
// Notes:

#include "adc.h"
#include "main.h"



#define ADC_INITTIMEOUT			7200000							//0.1s
//initialize SDADC peripheral
void adc_Init(void){
	int T;
	GPIO_InitTypeDef			GPIO_InitStructure;
	SDADC_InitTypeDef			SDADC_InitStructure;
	SDADC_AINStructTypeDef 		SDADC_AINStructure;

	RCC_APB2PeriphClockCmd(ADC_SDADCPERIPHCLK, ENABLE);			//enable peripheral clocks
	RCC_APB1PeriphClockCmd(RCC_APB1Periph_PWR, ENABLE);			//wut?
	RCC_AHBPeriphClockCmd(ADC_GPIOPERIPHCLK, ENABLE);

	SDADC_DeInit(ADC_ADC);

	GPIO_InitStructure.GPIO_Pin = ADC_POTMETVOLTAGE_PIN;
	GPIO_InitStructure.GPIO_Mode  = GPIO_Mode_AN;
	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
	GPIO_Init(ADC_POTMETVOLTAGE_PORT, &GPIO_InitStructure);

	PWR_SDADCAnalogCmd(ADC_ANALOGPWR, ENABLE);					//wake up analog part
	RCC_SDADCCLKConfig(RCC_SDADCCLK_SYSCLK_Div48);				//1.5MHz - 540KOhm input impedance
	SDADC_VREFSelect(SDADC_VREF_Ext);							//select reference at VREF pin
	for( T=0; T<(72000*4); T++ );								//72MHz -> 1ms = 72000... *4 = 4ms delay which is needed to stabilize reference
	SDADC_Cmd(SDADC3, ENABLE);									//enable sdadc peripheral

	SDADC_InitModeCmd(ADC_ADC, ENABLE);							//enter initialization mode

	T = ADC_INITTIMEOUT;
	while( (SDADC_GetFlagStatus(ADC_ADC, SDADC_FLAG_INITRDY) == RESET) && (--T != 0) );	//wait for INITRDY flag to be set
	if( T == 0 ){
		return;
	}

	//analog input configuration conf0: use single ended zero reference
	SDADC_AINStructure.SDADC_InputMode = SDADC_InputMode_SEZeroReference;
	SDADC_AINStructure.SDADC_Gain = SDADC_Gain_1;
	SDADC_AINStructure.SDADC_CommonMode = SDADC_CommonMode_VSSA;
	SDADC_AINStructure.SDADC_Offset = 0;
	SDADC_AINInit(ADC_ADC, SDADC_Conf_0, &SDADC_AINStructure);

	SDADC_ChannelConfig(ADC_ADC, ADC_ADCCHANNEL, SDADC_Conf_0);			//select channel with configuration above

	SDADC_InitStructure.SDADC_Channel = ADC_ADCCHANNEL;					//channel configuration
	SDADC_InitStructure.SDADC_ContinuousConvMode = DISABLE;
	SDADC_InitStructure.SDADC_FastConversionMode = DISABLE;
	SDADC_Init(ADC_ADC, &SDADC_InitStructure);

	SDADC_InitModeCmd(ADC_ADC, DISABLE);		  						//exit initialization mode

	SDADC_CalibrationSequenceConfig(ADC_ADC, SDADC_CalibrationSequence_1);	//configure calibration to be performed on conf0
	SDADC_StartCalibration(ADC_ADC);	//start MYADC_ADC Calibration

	T = ADC_INITTIMEOUT;
	while( (SDADC_GetFlagStatus(ADC_ADC, SDADC_FLAG_EOCAL) == RESET) && (--T != 0) );
	if( T == 0 ){

		return;
	}
 }



//read current value at SDADC input
int adc_GetValue(void){
	short int Value;
	SDADC_SoftwareStartConv(ADC_ADC);
	while( SDADC_GetFlagStatus(ADC_ADC, SDADC_FLAG_REOC) == RESET );	/* Wait Regular end of conversion flag to be set */
	Value = SDADC_GetConversionValue(ADC_ADC);
	return (int)Value;
}



//convert adc value into voltage
double adc_ConvertToVoltage(double adcValue){
	return (double)( ((double)adcValue + 32767) * (double)ADC_VREF / (double)ADC_RESOLUTION );
}

