/*
*Filename: mybuttons.c
*Author: Stanislav Subrt
*Year: 2013
*/

#include "buttons.h"
#include "main.h"



#define BTNS_QUEUESIZE	32
CircBufferStructTypedef PressedButtonStruct;
char PressedButtonQueue[BTNS_QUEUESIZE];

//////////////////////////////////////////////////////////////////////
////////// EXTERNAL FUNCTIONS ////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

void btns_Init(void){
	GPIO_InitTypeDef GPIOInitStruct;
	NVIC_InitTypeDef NVIC_InitStructure;
	EXTI_InitTypeDef EXTI_InitStructure;

	CB_Init(&PressedButtonStruct, PressedButtonQueue, BTNS_QUEUESIZE, '\n');	//initialize buffer for pressed buttons

	//enable peripheral clocks
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_SYSCFG, ENABLE);
	RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOB, ENABLE);

	//initialize LCD pins as pushpull outputs
	GPIOInitStruct.GPIO_Mode = GPIO_Mode_IN;
	GPIOInitStruct.GPIO_OType = GPIO_OType_PP;
	GPIOInitStruct.GPIO_Pin = BTNS_PINS_ALL;
	GPIOInitStruct.GPIO_PuPd = GPIO_PuPd_UP;
	GPIOInitStruct.GPIO_Speed = GPIO_Speed_Level_3;
	GPIO_Init(BTNS_PORT, &GPIOInitStruct);

	//connect BTN0 to EXTI
	SYSCFG_EXTILineConfig(BTNS_PORTSRC, BTNS_SRC_B0);
	SYSCFG_EXTILineConfig(BTNS_PORTSRC, BTNS_SRC_B1);
	SYSCFG_EXTILineConfig(BTNS_PORTSRC, BTNS_SRC_B2);

	//configure BTN0 release interrupt
	EXTI_InitStructure.EXTI_Line = EXTI_Line0;
	EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Interrupt;
	EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Rising;
	EXTI_InitStructure.EXTI_LineCmd = ENABLE;
	EXTI_Init(&EXTI_InitStructure);

	//configure BTN1 release interrupt
	EXTI_InitStructure.EXTI_Line = EXTI_Line1;
	EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Interrupt;
	EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Rising;
	EXTI_InitStructure.EXTI_LineCmd = ENABLE;
	EXTI_Init(&EXTI_InitStructure);

	//configure BTN2 release interrupt
	EXTI_InitStructure.EXTI_Line = EXTI_Line2;
	EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Interrupt;
	EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Rising;
	EXTI_InitStructure.EXTI_LineCmd = ENABLE;
	EXTI_Init(&EXTI_InitStructure);

	/* Enable and set EXTI0 Interrupt to the lowest priority */
	NVIC_InitStructure.NVIC_IRQChannel = EXTI0_IRQn;
	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0x3;
	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0x03;
	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
	NVIC_Init(&NVIC_InitStructure);

	/* Enable and set EXTI1 Interrupt to the lowest priority */
	NVIC_InitStructure.NVIC_IRQChannel = EXTI1_IRQn;
	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0x3;
	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0x3;
	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
	NVIC_Init(&NVIC_InitStructure);

	/* Enable and set EXTI2 Interrupt to the lowest priority */
	NVIC_InitStructure.NVIC_IRQChannel = EXTI2_TS_IRQn;
	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0x3;
	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0x3;
	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
	NVIC_Init(&NVIC_InitStructure);


	btns_ClearQueue();
}


int btns_GetPressedID(void){
	int PressedButtonID = CB_RemoveTail(&PressedButtonStruct);

	if( PressedButtonID == CB_BUFFER_EMPTY){
		return BTNS_NONE;
	}
	else{
		return PressedButtonID;
	}
}

void btns_PressBtnByID(int btnID){
	NVIC_DisableIRQ(USART1_IRQn);
	CB_Add(&PressedButtonStruct, btnID);
	NVIC_EnableIRQ(USART1_IRQn);
}


void btns_ClearQueue(void){
	while ( CB_RemoveTail(&PressedButtonStruct) != CB_BUFFER_EMPTY );
}

//////////////////////////////////////////////////////////////////////
////////// INTERNAL FUNCTIONS ////////////////////////////////////////
//////////////////////////////////////////////////////////////////////


void EXTI0_IRQHandler(void){
	if(EXTI_GetITStatus(EXTI_Line0) == SET){
		CB_Add(&PressedButtonStruct, BTNS_ID_BTN0);
		EXTI_ClearITPendingBit(EXTI_Line0);
	}
}


void EXTI1_IRQHandler(void){
	if(EXTI_GetITStatus(EXTI_Line1) == SET){
		CB_Add(&PressedButtonStruct, BTNS_ID_BTN1);
		EXTI_ClearITPendingBit(EXTI_Line1);
	}
}


void EXTI2_TS_IRQHandler(void){
	if(EXTI_GetITStatus(EXTI_Line2) == SET){
		CB_Add(&PressedButtonStruct, BTNS_ID_BTN2);
		EXTI_ClearITPendingBit(EXTI_Line2);
	}
}
