// Header:
// File Name:						myusart.c
// File Description:
// Author:							Stanislav Subrt
// Date:							2013
// Todo:
// Notes:

#include "comm.h"
#include "main.h"



#define RXBUFFERSIZE	512		//be super sure this is power of two number -> 64, 128, 256, 512, 1024 etc
#define TXBUFFERSIZE	4096

//RX buffer
char RXBufferStorage[RXBUFFERSIZE];
CircBufferStructTypedef RXBuffer;

//TX buffer
char TXBufferStorage[TXBUFFERSIZE];
CircBufferStructTypedef TXBuffer;

//Last command buffer
char CmdRXBuffer[RXBUFFERSIZE];



//initialize all usart peripherals
void comm_Init(void){
    NVIC_InitTypeDef NVIC_InitStruct;
    GPIO_InitTypeDef GPIO_InitStruct;
    USART_InitTypeDef USART_InitStruct;


	//init RX and TX circular buffers
	CB_Init(&RXBuffer, RXBufferStorage, RXBUFFERSIZE, '\n');
	CB_Init(&TXBuffer, TXBufferStorage, TXBUFFERSIZE, '\n');


    //enable the gloabal Interrupt
    NVIC_InitStruct.NVIC_IRQChannel = USART1_IRQn;
    NVIC_InitStruct.NVIC_IRQChannelPreemptionPriority = 0;
    NVIC_InitStruct.NVIC_IRQChannelCmd = ENABLE;
    NVIC_Init(&NVIC_InitStruct);


    //configure gpio pins for usart
    RCC_AHBPeriphClockCmd(USART1_PORTPERIPHCLK, ENABLE);
    RCC_APB2PeriphClockCmd(USART1_PERIPHCLK, ENABLE);


    GPIO_InitStruct.GPIO_Pin  = USART1_TXPIN;
    GPIO_InitStruct.GPIO_Mode = GPIO_Mode_AF;
    GPIO_InitStruct.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_InitStruct.GPIO_OType = GPIO_OType_PP;
    GPIO_InitStruct.GPIO_PuPd = GPIO_PuPd_UP;

    GPIO_Init(USART1_PORT, &GPIO_InitStruct);
    GPIO_PinAFConfig(USART1_PORT, USART1_TXPINSOURCE, GPIO_AF_7);


    GPIO_InitStruct.GPIO_Pin  = USART1_RXPIN;
    GPIO_InitStruct.GPIO_Mode = GPIO_Mode_AF;
    GPIO_InitStruct.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_InitStruct.GPIO_OType = GPIO_OType_PP;
    GPIO_InitStruct.GPIO_PuPd = GPIO_PuPd_UP;

    GPIO_Init(USART1_PORT, &GPIO_InitStruct);
    GPIO_PinAFConfig(USART1_PORT, USART1_RXPINSOURCE, GPIO_AF_7);


    USART_InitStruct.USART_BaudRate = 115200;
    USART_InitStruct.USART_WordLength = USART_WordLength_8b;
    USART_InitStruct.USART_StopBits = USART_StopBits_1;
    USART_InitStruct.USART_Parity = USART_Parity_No;
    USART_InitStruct.USART_Mode = USART_Mode_Rx | USART_Mode_Tx;
    USART_InitStruct.USART_HardwareFlowControl = USART_HardwareFlowControl_None;
    USART_Init(USART1, &USART_InitStruct);
	USART_Cmd(USART1, ENABLE);

	USART_ITConfig(USART1, USART_IT_RXNE, ENABLE);							//enable RX not empty interrupt
	USART_ClearITPendingBit(USART1, USART_IT_RXNE);

}



//add TX data into transmit queue and start transmitting
void comm_SendData(char *dataToSend, int lengthInBytes){

	if( CB_AddMultiple(&TXBuffer, dataToSend, lengthInBytes) == CB_BUFFER_FULL){
		//while(1);
	}
	USART_ITConfig(USART1_PERIPH, USART_IT_TXE, ENABLE);	//enable TX empty interrupt which will send all data
}



//handle received commands
void comm_HandleReceivedData(void){
	char FirstCharOfPacket, SecondCharOfPacket;
	char tmp[128];
	char *pApertureStatus;

	if( CB_IsSpecialCharReceived(&RXBuffer) ){
		CB_RemoveExistingUpToSpecChar(&RXBuffer, CmdRXBuffer);
		FirstCharOfPacket = CmdRXBuffer[0];
		SecondCharOfPacket = CmdRXBuffer[1];

		switch (FirstCharOfPacket){

			case 'i':
				sprintf(tmp, "<i 1\r\n>");
				comm_SendData(tmp, strlen(tmp));
				comm_SendData(VERSIONSTRING, sizeof(VERSIONSTRING));
			break;

			case 'n':
				sprintf(tmp, "<n 1\r\n>");
				comm_SendData(tmp, strlen(tmp));
				comm_SendData(DEVICENAMESTRING, sizeof(DEVICENAMESTRING));
			break;

			case 'g':
				if(SecondCharOfPacket == 'd'){
					pm_SendLatestCycleResults(pm_GetLatestCycle());
				}
				else if(SecondCharOfPacket == 's'){
					pApertureStatus = pm_GetApertureStatus();
					comm_SendData(pApertureStatus, strlen(pApertureStatus));
				}
			break;

			case 'r':
				if(SecondCharOfPacket == 'd'){
					pm_ClearLatestCycleData();
					gui_RequestRefresh();
				}
			break;





			case 'c':
				if( SecondCharOfPacket == '0' )		pm_SelectChannel(0);		//keep same channel
				else if(SecondCharOfPacket == '1') 	pm_SelectChannel(1);
				else if(SecondCharOfPacket == '2')	pm_SelectChannel(2);
				else if(SecondCharOfPacket == '3')	pm_SelectChannel(3);
				else if(SecondCharOfPacket == '4')	pm_SelectChannel(4);
				else 								pm_SelectChannel(-1);		//switch off all channels
			break;

			case 'p':
				if(SecondCharOfPacket=='0'){
					sd_SetPolarity(0);
				}
				else if(SecondCharOfPacket=='1'){
					sd_SetPolarity(1);
				}
				else {
					sd_TogglePolarity();
				}
			break;

			case 't':
				sd_SetPeriod( ( (CmdRXBuffer[1] - 0x30) * 1000 ) + ( (CmdRXBuffer[2] - 0x30) * 100 ) );
			break;


			case 'b':
				//software button pressing
				if(SecondCharOfPacket == '0'){
					btns_PressBtnByID(BTNS_ID_BTN0);
				}
				else if(SecondCharOfPacket == '1'){
					btns_PressBtnByID(BTNS_ID_BTN1);
				}
				else if(SecondCharOfPacket == '2'){
					btns_PressBtnByID(BTNS_ID_BTN2);
				}
			break;

			default:
				comm_SendData("Command does not exist!\r\n", 42);
			break;
		}
	}
}


//////////////////////////////////////////////////////////////////////
////////// INTERNAL FUNCTIONS ////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

//interrupt handler
void USART1_IRQHandler(void){
	int tmp;
	char ReceivedChar;

	//Handle received char
	if(USART_GetITStatus(USART1, USART_IT_RXNE)==SET){						//RX Not Empty interrupt
		ReceivedChar = (char)USART_ReceiveData(USART1);
		CB_Add(&RXBuffer, ReceivedChar);
	}

	//handle char to transmit
	else if(USART_GetITStatus(USART1, USART_IT_TXE)==SET){						//TX empty
		tmp = CB_RemoveTail(&TXBuffer);							//read character to send
		if( tmp != CB_BUFFER_EMPTY){										//if TX buffer contains chars to send
			USART_SendData(USART1, (char)tmp);
		}
		else{																//tx buffer is empty
			USART_ITConfig(USART1, USART_IT_TXE, DISABLE);
			USART_ClearITPendingBit(USART1, USART_IT_TXE);					//TXE int is cleared by SendData function or here manually
		}
	}															//not managed interrupt

	else{//if(USART_GetITStatus(USART1, USART_IT_ORE)==SET){						//overrun error just ignore now
		USART_ClearITPendingBit(USART1, USART_IT_ORE);
		USART_ClearITPendingBit(USART1, USART_IT_EOB);

	}
}
