// Header:
// File Name:					mysyndet.c
// File Description:
// Author:						Stanislav Subrt
// Date:							2013
// Todo:
// Notes:

#include "syndet.h"
#include "main.h"



//////////////////////////////////////////////////////////////////////
////////// FUNCTION PROTOTYPES ///////////////////////////////////////
//////////////////////////////////////////////////////////////////////

static void sd_StartCountingSyndet(void);
//static void sd_StopCountingSyndet(void);
static void sd_Advance(void);



#define PHASECOUNT	4
typedef struct{
	unsigned Period;							//in ms

	int Enabled;
	int Polarity;
	int PhaseCount;
	int CurPhase;
	int Fm[PHASECOUNT];
	int SD1[PHASECOUNT];
	int SD2[PHASECOUNT];
} SyndetTd;



SyndetTd McuSyndetInfo = {
	1000,					//period in ms
	0,
	0,						//polarity
	4,						//phase count
	0,						//cur phase
	{1, 1, 0, 0},			//Fm function
	{1, 0, 1, 1},			//SD1 function
	{0, 0, 0, 1}			//SD2 function
};



void sd_Init(void){
	GPIO_InitTypeDef			GPIO_InitStructure;
	NVIC_InitTypeDef 			NVIC_InitStructure;
	//TIM_TimeBaseInitTypeDef 	TIM_TimeBaseInitStruct;

	//gpio init - PD13 = fm, PD14 = SD1, PD15 = SD2
	RCC_AHBPeriphClockCmd(SYNDET_PORT_PERIPHCLK, ENABLE);

	GPIO_InitStructure.GPIO_Pin  = SYNDET_PINS;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_2MHz;
	GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
	GPIO_Init(SYNDET_PORT, &GPIO_InitStructure);

	//tim2 clock enable for syndet period counting - params are being set just before starting timer
	RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM2, ENABLE);

	//configuretimer interrupts
	NVIC_InitStructure.NVIC_IRQChannel = TIM2_IRQn;
	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 1;
	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
	NVIC_Init(&NVIC_InitStructure);
	TIM_ClearITPendingBit(TIM2, TIM_IT_Update);
	TIM_ITConfig(TIM2, TIM_IT_Update, ENABLE);

	//start syndet
	sd_Start();
}



#define ONE_MS_CNT_TIM2		(unsigned) 7200
#define MINPERIODMS				1
#define MAXPERIODMS				( 0xFFFFFFFF / ONE_MS_CNT_TIM2 - 1000)
void sd_Start(void){
	McuSyndetInfo.CurPhase = 0;
	McuSyndetInfo.Enabled = 1;
	sd_StartCountingSyndet();
	sd_Advance();
}



void sd_SetPolarity(int pol){
	if ( pol == 1 )		McuSyndetInfo.Polarity = 1;
	else if ( pol == 0) McuSyndetInfo.Polarity = 0;
	else return;

}



void sd_TogglePolarity(void){
	McuSyndetInfo.Polarity = McuSyndetInfo.Polarity ^ 1;
}



void sd_SetPeriod(int period){

	if( McuSyndetInfo.Period == period  ) return;	//&& McuSyndetInfo.Enabled == 1

	//check period setup
	if( period == 0){
		McuSyndetInfo.Enabled = 0;
		McuSyndetInfo.CurPhase = 1;
		McuSyndetInfo.Period = MINPERIODMS;
		sd_StartCountingSyndet();
		return;
	}
	else if( period < MINPERIODMS )		period = MINPERIODMS;
	else if ( period > MAXPERIODMS ) 	period = MAXPERIODMS;
	McuSyndetInfo.Enabled = 1;
	McuSyndetInfo.CurPhase = 0;
	McuSyndetInfo.Period = period;
	sd_StartCountingSyndet();
}



//////////////////////////////////////////////////////////////////////
////////// INTERNAL FUNCTIONS ////////////////////////////////////////
//////////////////////////////////////////////////////////////////////


//ISR for next time step of syn det
static void sd_Advance(void){

	if( McuSyndetInfo.Enabled ){
		McuSyndetInfo.CurPhase++;

		if( McuSyndetInfo.CurPhase > ( McuSyndetInfo.PhaseCount - 1 ) ) {
			McuSyndetInfo.CurPhase = 0;
		}
	}

	if ( ( McuSyndetInfo.Fm[McuSyndetInfo.CurPhase] ) ^ (McuSyndetInfo.Polarity) )		GPIO_SetBits(SYNDET_PORT, SYNDET_PIN_FM);		//polarity bit current phase xor polaritytogglebit; change polarity of syn det
	else GPIO_ResetBits(SYNDET_PORT, SYNDET_PIN_FM);

	if ( McuSyndetInfo.SD1[McuSyndetInfo.CurPhase] )	GPIO_SetBits(SYNDET_PORT, SYNDET_PIN_SD1);
	else GPIO_ResetBits(SYNDET_PORT, SYNDET_PIN_SD1);


	if ( McuSyndetInfo.SD2[McuSyndetInfo.CurPhase] )	GPIO_SetBits(SYNDET_PORT, SYNDET_PIN_SD2);
	else GPIO_ResetBits(SYNDET_PORT, SYNDET_PIN_SD2);
}



void TIM2_IRQHandler(void){
	if(TIM_GetITStatus(TIM2, TIM_IT_Update) == SET){
		sd_Advance();
		TIM_ClearITPendingBit(TIM2, TIM_IT_Update);
	}
}



static void sd_StartCountingSyndet(void){
	TIM_TimeBaseInitTypeDef 	TIM_TimeBaseInitStruct;

	TIM_TimeBaseInitStruct.TIM_Prescaler = 9;		//coreclk/(9+1) = timerclk
	TIM_TimeBaseInitStruct.TIM_CounterMode = TIM_CounterMode_Up;
	TIM_TimeBaseInitStruct.TIM_Period = ( ( McuSyndetInfo.Period * ONE_MS_CNT_TIM2 ) / PHASECOUNT );
	TIM_TimeBaseInitStruct.TIM_ClockDivision = TIM_CKD_DIV1;
	TIM_TimeBaseInitStruct.TIM_RepetitionCounter = 0;
	TIM_TimeBaseInit(TIM2, &TIM_TimeBaseInitStruct);

	TIM_Cmd(TIM2, ENABLE);
}


/*
static void sd_StopCountingSyndet(void){
		TIM_Cmd(TIM2, DISABLE);
		TIM_SetCounter(TIM2, 0);
}
*/
