/*
*Filename: mygui.c
*Author: Stanislav Subrt
*Year: 2013
*/

#include "mygui.h"
#include "mylcd.h"
#include "mybuttons.h"
#include "mypotmet.h"
#include "myusart.h"

#include <stdio.h>
#include <string.h>
#include <math.h>

//////////////////////////////////////////////////////////////////////
////////// INTERNAL TYPEDEFS /////////////////////////////////////////
//////////////////////////////////////////////////////////////////////


typedef struct{
	char *BtnCaption;
	int BtnEventID;					//ID of event that should happen after click
} BtnStructTypedef;



typedef struct {
	char ScreenTitle[30];
	void (*RedrawWindow)();		//pointer to function to draw/redraw screen
	BtnStructTypedef Btn0;
	BtnStructTypedef Btn1;
	BtnStructTypedef Btn2;
} ScreenStructTypedef;



typedef struct {
	ScreenStructTypedef *ScreenListArray;
	int NumberOfScreensInList;
	int ActiveScreenIndex;
	int IsRefreshNeeded;
} DispStructTypedef;




//////////////////////////////////////////////////////////////////////
////////// INTERNAL DECLARATIONS /////////////////////////////////////
//////////////////////////////////////////////////////////////////////

static void mygui_DrawScreenButtons(char* screenBuffer, ScreenStructTypedef* screenStruct);
static void mygui_DrawTitleAndBtns(void);
static void mygui_DrawMeasurementProgress(void);
static void mygui_DrawRunningMeasurementValues(int isDC);

static void mygui_ShowNextScreen(void);

static int mygui_GetBtnEventID(BtnStructTypedef *btnStruct);
static void mygui_CallEventByID(int eventID);



static void DCTestScreenRedraw(void);
static void DCCrackDetectionScreenRedraw(void);
static void DCCrackMeasurementScreenRedraw(void);



#define STRUCTCOUNT_IN_ARRAY(x)	( (sizeof(x)) / (sizeof(x[0])) )
#define DEFAULT_SCREEN_INDEX	0

#define REFRESH_NEEDED			1
#define REFRESH_NOT_NEEDED		0


//image buffer for our display
char ScreenBufferArray[MYLCD_SCREENBUFFERSIZE];
///////////////////////////////////////////////////////////////////////////////////
////////// HERE DEFINE EACH SCREEN STRUCT AND ADD INTO DISPSTRUCT /////////////////
///////////////////////////////////////////////////////////////////////////////////

//image buffer for our display
char ScreenBufferArray[MYLCD_SCREENBUFFERSIZE];

//event IDs - every button pressed has specified EventID for active screen
#define MYGUI_EVENTID_NONE						-1
#define MYGUI_EVENTID_STOPALL					0
#define MYGUI_EVENTID_CHANGEMODE				1
#define MYGUI_EVENTID_STARTDCREF				2
#define MYGUI_EVENTID_STARTDCTEST				3
#define MYGUI_EVENTID_STARTDCCRACKDETECTION		4
#define MYGUI_EVENTID_STARTDCCRACKMEASUREMENT	6


//SCREENSTRUCTS - screentitle cannot be longer than 20 chars
#define DCTESTSCREENSTRUCT				{"DC TEST", 				DCTestScreenRedraw,						{"Start", MYGUI_EVENTID_STARTDCTEST}, 				{"", MYGUI_EVENTID_NONE}, 				{"Mode", MYGUI_EVENTID_CHANGEMODE} }
#define DCCRACKDETECTIONSTRUCT			{"DC CRACK DETECT",			DCCrackDetectionScreenRedraw,			{"Start", MYGUI_EVENTID_STARTDCCRACKDETECTION}, 	{"Ref", MYGUI_EVENTID_STARTDCREF}, 		{"Mode", MYGUI_EVENTID_CHANGEMODE} }
#define DCCRACKMEASUREMENTSTRUCT		{"DC CRACK MEASUREMENT",	DCCrackMeasurementScreenRedraw,			{"Start", MYGUI_EVENTID_STARTDCCRACKMEASUREMENT}, 	{"", MYGUI_EVENTID_NONE}, 				{"Mode", MYGUI_EVENTID_CHANGEMODE} }

//SCREENSTRUCT LIST
ScreenStructTypedef ScreenList[] = {DCTESTSCREENSTRUCT, DCCRACKDETECTIONSTRUCT, DCCRACKMEASUREMENTSTRUCT};	// 	//screen structs aded into array

//DISPSTRUCT
DispStructTypedef DispStruct = {ScreenList, STRUCTCOUNT_IN_ARRAY(ScreenList), DEFAULT_SCREEN_INDEX, REFRESH_NEEDED};


//////////////////////////////////////////////////////////////////////
////////// EXTERNAL FUNCTIONS ////////////////////////////////////////
//////////////////////////////////////////////////////////////////////


void mygui_Init(void){
	mylcd_Init();
	mybuttons_Init();
}


void mygui_RequestLCDRefresh(void){
	DispStruct.IsRefreshNeeded = 1;
}


#define DEBUGBTNSx
void mygui_HandleRefreshRequests(void){
	int PressedButton;
	int EventID;

	while( (PressedButton = mybuttons_GetPressedID()) != MYBUTTONS_NONE ){	//read all pressed buttons from queue
		switch( PressedButton ){											//for each of pressed buttons select proper event ID
			case MYBUTTONS_ID_BTN0:
				EventID = mygui_GetBtnEventID( &(DispStruct.ScreenListArray[DispStruct.ActiveScreenIndex].Btn0) );
				#ifdef DEBUGBTNS
					myusart_SendData(USART1, "Btn0 pressed\n\r", 14);
				#endif
			break;

			case MYBUTTONS_ID_BTN1:
				EventID = mygui_GetBtnEventID( &(DispStruct.ScreenListArray[DispStruct.ActiveScreenIndex].Btn1) );
				#ifdef DEBUGBTNS
					myusart_SendData(USART1, "Btn1 pressed\n\r", 14);
				#endif
			break;

			case MYBUTTONS_ID_BTN2:
				EventID = mygui_GetBtnEventID( &(DispStruct.ScreenListArray[DispStruct.ActiveScreenIndex].Btn2) );
				#ifdef DEBUGBTNS
					myusart_SendData(USART1, "Btn2 pressed\n\r", 14);
				#endif
			break;
		}
		mygui_CallEventByID(EventID);										//call event by ID
	}

	if( DispStruct.IsRefreshNeeded ){

		DispStruct.ScreenListArray[DispStruct.ActiveScreenIndex].RedrawWindow();	//call active screens redraw function

		mylcd_UpdateScreen(ScreenBufferArray);
		DispStruct.IsRefreshNeeded = 0;
	}
}
//////////////////////////////////////////////////////////////////////
////////// BUTTON CALLBACKS AND REDRAW FUNCTIONS /////////////////////
//////////////////////////////////////////////////////////////////////


//Measurement screen

static void DCTestScreenRedraw(void){
	char tmp[500];
	float Ch1P, Ch1M, Ch2P, Ch2M, Sub1, Sub2;

	BytePositionStructTypedef StartPos = {0, 2};
	MeasurementResultsStructTypedef* pResult;

	mygui_DrawTitleAndBtns();
	mygui_DrawMeasurementProgress();

	//get pointer to measurement results struct of currently running or ready measurement
	pResult = mypotmet_GetCurrentMeasurementResults(mypotmet_GetRunningCycle());	//get values from running measuremen
	if( pResult == 0 ) {

		pResult =  mypotmet_GetCurrentMeasurementResults(mypotmet_GetLatestCycle());
		if( pResult == 0 ){
			return;
		}
	}


	//draw test results from channel 1 pol+ pol- sub/2
	//draw test results from channel 2 pol+ pol- sub/2

	Ch2M = pResult->MeasResultsArray[3];
	Ch2P = pResult->MeasResultsArray[2];
	Ch1M = pResult->MeasResultsArray[1];
	Ch1P = pResult->MeasResultsArray[0];

	switch(pResult->ReadyIndex){
		case 4:
			Sub2 = (Ch2P-Ch2M)/2;

			StartPos.x = 1*128/3;
			StartPos.page = 3;
			sprintf(tmp, "%.1f", Ch2M);
			mylcd_PutStr(ScreenBufferArray, tmp, strlen(tmp), &StartPos, 0);

			StartPos.x = 2*128/3;
			StartPos.page = 3;
			sprintf(tmp, "%.1f", Sub2);
			mylcd_PutStr(ScreenBufferArray, tmp, strlen(tmp), &StartPos, 0);


			//evaluate test
			if( abs((int)Ch2M) < 1000 && abs((int)Ch2P) < 1000 && abs((int)Ch1M) < 1000 && abs((int)Ch1P) < 1000){
				sprintf(tmp, "Gain too low!");
			}
			else if( abs((int)Ch2M) > 3000 || abs((int)Ch2P) > 3000 || abs((int)Ch1M) > 3000 || abs((int)Ch1P) > 3000){
				sprintf(tmp, "Gain too high!");
			}
			else{
				sprintf(tmp, "Gain OK!");
			}
			StartPos.x = 0;
			StartPos.page = 4;
			mylcd_PutStr(ScreenBufferArray, tmp, strlen(tmp), &StartPos, 0);


			if( ((Ch1P + Ch1M)/2 + (Ch2P + Ch2M)/2) > 2000 ||  ( (Ch1P + Ch1M)/2 + (Ch2P + Ch2M)/2 ) < 2000 ){
				sprintf(tmp, "Tune zero!");
			}
			StartPos.x = 0;
			StartPos.page = 5;
			mylcd_PutStr(ScreenBufferArray, tmp, strlen(tmp), &StartPos, 0);

		case 3:
			StartPos.x = 0;
			StartPos.page = 3;
			sprintf(tmp, "%.1f", Ch2P);
			mylcd_PutStr(ScreenBufferArray, tmp, strlen(tmp), &StartPos, 0);

		case 2:
			Sub1 = (Ch1P-Ch1M)/2;

			StartPos.x = 1*128/3;
			StartPos.page = 2;
			sprintf(tmp, "%.1f", Ch1M);
			mylcd_PutStr(ScreenBufferArray, tmp, strlen(tmp), &StartPos, 0);

			StartPos.x = 2*128/3;
			StartPos.page = 2;
			sprintf(tmp, "%.1f", Sub1);
			mylcd_PutStr(ScreenBufferArray, tmp, strlen(tmp), &StartPos, 0);

		case 1:
			StartPos.x = 0;
			StartPos.page = 2;
			sprintf(tmp, "%.1f", Ch1P);
			mylcd_PutStr(ScreenBufferArray, tmp, strlen(tmp), &StartPos, 0);
	}

	mygui_RequestLCDRefresh();

}



static void DCCrackDetectionScreenRedraw(void){
	mygui_DrawTitleAndBtns();
	mygui_DrawMeasurementProgress();



	mygui_RequestLCDRefresh();

}



static void DCCrackMeasurementScreenRedraw(void){
	mygui_DrawTitleAndBtns();
	mygui_DrawMeasurementProgress();



	mygui_RequestLCDRefresh();
}



//////////////////////////////////////////////////////////////////////
////////// INTERNAL FUNCTIONS ////////////////////////////////////////
//////////////////////////////////////////////////////////////////////


static void mygui_ShowNextScreen(void){

	DispStruct.ActiveScreenIndex++;
	if( DispStruct.ActiveScreenIndex >= DispStruct.NumberOfScreensInList ){	//if index after increment points behind list
		DispStruct.ActiveScreenIndex = 0;
	}
	mygui_RequestLCDRefresh();
}

static void mygui_DrawTitleAndBtns(void){
	BytePositionStructTypedef StartPos = {0, 0};
	BytePositionStructTypedef EndPos = {127, 0};
	int i;

	//place title bar
	mylcd_Fill(ScreenBufferArray, 0);		//clear screen
	mylcd_PlaceBackground(ScreenBufferArray, &StartPos, &EndPos, 0xFF);
	i = strlen(DispStruct.ScreenListArray[DispStruct.ActiveScreenIndex].ScreenTitle);
	if( i>20 ) i = 20;
	mylcd_PutStr(ScreenBufferArray, DispStruct.ScreenListArray[DispStruct.ActiveScreenIndex].ScreenTitle, i, &StartPos, 1);	//add screen caption with inverted colors to fit placed background


	//draw buttons
	mygui_DrawScreenButtons(ScreenBufferArray, &(DispStruct.ScreenListArray[DispStruct.ActiveScreenIndex]));


	//print column captions
	StartPos.x = 0;
	StartPos.page = 1;
	mylcd_PutStr(ScreenBufferArray, "POL+", 4, &StartPos, 0);

	StartPos.x = 1*128/3;
	StartPos.page = 1;
	mylcd_PutStr(ScreenBufferArray, "POL-", 4, &StartPos, 0);

	StartPos.x = 2*128/3;
	StartPos.page = 1;
	mylcd_PutStr(ScreenBufferArray, "SUB/2", 4, &StartPos, 0);
}




static void mygui_DrawMeasurementProgress(void){
	BytePositionStructTypedef StartPos = {0, 0};
	int CycleProgress;
	char tmp[50];

	//place measurement progress indicator
	StartPos.x = 0;
	StartPos.page = 6;

	CycleProgress = mypotmet_GetRunningCycleProgress();			//percentage

	if(CycleProgress == MYPOTMET_CYCLE_NOT_RUNNING){
		sprintf(tmp, "Aperture ready!");
	}
	else{
		sprintf(tmp, "Measuring!");
	}
	mylcd_PutStr(ScreenBufferArray, tmp, strlen(tmp), &StartPos, 0);
}



static void mygui_DrawRunningMeasurementValues(int isDC){
	char tmp[500];
	int i, ValuesOnPage = 0;
	BytePositionStructTypedef StartPos = {0, 2};
	MeasurementResultsStructTypedef* pResult;

	//place measurement values
	pResult = mypotmet_GetCurrentMeasurementResults(mypotmet_GetRunningCycle());	//get values from running measuremen
	if( pResult == 0 ) {

		pResult =  mypotmet_GetCurrentMeasurementResults(mypotmet_GetLatestCycle());
		if( pResult == 0 ){
			return;
		}
	}
	if( isDC ){
		for( i=1; i<=(pResult->ReadyIndex); i++ ){	//for each ready channel value from running measurement	- readyindex = 0 if no valid measurement now
			sprintf(tmp, "%06.1lf", pResult->MeasResultsArray[i-1]);	//create string from double


			if( ( i&0x1 ) == 1  ){	//
				StartPos.x = 30;
			}
			else{
				StartPos.x = 80;
			}

			if( ValuesOnPage == 2){
				ValuesOnPage = 0;
				StartPos.page++;
			}

			mylcd_PutStr(ScreenBufferArray, tmp, strlen(tmp), &StartPos, 0);	//add screen caption
			ValuesOnPage++;

		}
	}
	else{
		for( i=1; i<=(pResult->ReadyIndex); i++ ){	//for each ready channel value from running measurement	- readyindex = 0 if no valid measurement now
			sprintf(tmp, "%06.1lf", pResult->MeasResultsArray[i-1]);	//create string from double
			if( i<5 ){	//first four channels (0 is no valid data, 1...4 is pol+ ch1-ch4, 5...8 is pol- ch1-ch4
				StartPos.x = 30;
				StartPos.page = i + 2 -1;		//2-1...base page, i... vertical position
				mylcd_PutStr(ScreenBufferArray, tmp, strlen(tmp), &StartPos, 0);	//add screen caption
			}
			else{		//second for channels - toggled polarity
				StartPos.x = 80;
				StartPos.page = i + 2 - 4 - 1;
				mylcd_PutStr(ScreenBufferArray, tmp, strlen(tmp), &StartPos, 0);	//add screen caption
			}
		}
	}
}


static void mygui_DrawScreenButtons(char* screenBuffer, ScreenStructTypedef* screenStruct){
	BytePositionStructTypedef Pos;
	BytePositionStructTypedef BgStart = {0, 7};
	BytePositionStructTypedef BgEnd = {127, 7};


	mylcd_PlaceBackground(screenBuffer, &BgStart, &BgEnd, 0xFF);

	Pos.page = 7;
	Pos.x = 0;
	mylcd_PutStr(screenBuffer, screenStruct->Btn0.BtnCaption, strlen(screenStruct->Btn0.BtnCaption), &Pos, 1);

	Pos.x = 1*128/3;
	mylcd_PutStr(screenBuffer, screenStruct->Btn1.BtnCaption, strlen(screenStruct->Btn1.BtnCaption), &Pos, 1);

	Pos.x = 2*128/3;
	mylcd_PutStr(screenBuffer, screenStruct->Btn2.BtnCaption, strlen(screenStruct->Btn2.BtnCaption), &Pos, 1);

	//mylcd_InvertPixels(screenBuffer, &InvStart, &InvEnd);
}


static int mygui_GetBtnEventID(BtnStructTypedef *btnStruct){
	return btnStruct->BtnEventID;
}


static void mygui_CallEventByID(int eventID){
	switch (eventID){
		case MYGUI_EVENTID_STOPALL:
			mypotmet_StopMeasurementCycle();
			mygui_RequestLCDRefresh();
		break;

		case MYGUI_EVENTID_CHANGEMODE:
			mypotmet_StopMeasurementCycle();
			mygui_ShowNextScreen();
//			mypotmet_ClearLatestStorageData();
			mypotmet_ClearLatestCycleData();
			mygui_RequestLCDRefresh();
		break;




		case MYGUI_EVENTID_STARTDCTEST:
			mypotmet_StartMeasurementCycle(&mypotmet_DCTestMeasCycle);
			mygui_RequestLCDRefresh();
		break;
/*
		case MYGUI_EVENTID_STARTDCMEASUREMENT:
			mypotmet_StartMeasurementCycle(&DCMeasurementCycleStruct);
			mygui_RequestLCDRefresh();
		break;

		case MYGUI_EVENTID_STARTMEASUREMENTLONG:
			mypotmet_StartMeasurementCycle(&DCMeasurementLongCycleStruct);
			mygui_RequestLCDRefresh();
		break;

		case MYGUI_EVENTID_STARTDCTEST:
			mypotmet_StartMeasurementCycle(&DCTestMeasurementCycleStruct);
			mygui_RequestLCDRefresh();
		break;

		case MYGUI_EVENTID_STARTTEST:
			mypotmet_StartMeasurementCycle(&TestMeasurementCycleStruct);
			mygui_RequestLCDRefresh();
		break;
*/
	}
}
